package treeplus.visualization;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JLabel;

import prefuse.Constants;
import prefuse.render.LabelRenderer;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import treeplus.visualization.demo.ITreePlusRunner;

/**
 * {@link LabelRenderer}, that determines a part of the appearance of the nodes
 * in the tree. The Renderer draws the arrow images (incoming, outgoing and
 * bidirectional arrows) and the bars which visualize the number of adjacent
 * nodes of one node.
 * 
 * @author Neubauer, Kstinger
 * 
 */
public class BarLabelRenderer extends LabelRenderer {

	private ITreePlusRunner runner;
	private boolean barsDisplayed = false;

	private double arcWidth = 8.0;
	private double arcHeight = 8.0;
	private double defaultWidth = 200;
	private int strokeThickness = 2;
	private int barDistance = 3;
	private int barDistanceToText = 5;

	private BufferedImage imgRedRight;
	private BufferedImage imgRedLeft;
	private BufferedImage imgBlueRight;
	private BufferedImage imgBlueLeft;
	private BufferedImage imgBidirectional;

	/**
	 * Creates the BarLabelRenderer using the given {@link TreePlusRunner}.
	 * 
	 * @param runner
	 *            the current {@link TreePlusRunner} used to get information
	 *            about the visualization
	 */
	public BarLabelRenderer(ITreePlusRunner runner) {
		this(runner, 8, 8, 2, 3, 200);
	}

	/**
	 * Creates the BarLabelRenderer using the given {@link TreePlusRunner} with
	 * the given width and height of the arcs, the distance between the bars,
	 * the thickness of the stroke and the default width of the bars.
	 * 
	 * @param runner
	 *            the current {@link TreePlusRunner} used to get information
	 *            about the visualization
	 * @param arcWidth
	 *            the width of the arcs
	 * @param arcHeight
	 *            the height of the arcs
	 * @param strokeThickness
	 *            the thickness of the bars
	 * @param barDistance
	 *            the distance between the bars
	 * @param defaultBarWidth
	 *            the width of the bars
	 */
	public BarLabelRenderer(ITreePlusRunner runner, double arcWidth,
			double arcHeight, int strokeThickness, int barDistance,
			double defaultBarWidth) {
		super(runner.getLabel());
		this.runner = runner;

		this.arcWidth = arcWidth;
		this.arcHeight = arcHeight;
		this.strokeThickness = strokeThickness;
		this.barDistance = barDistance;
		this.defaultWidth = defaultBarWidth;

		try {
			imgRedRight = loadImage(runner.getRedRightFile());
			imgRedLeft = loadImage(runner.getRedLeftFile());

			imgBlueRight = loadImage(runner.getBlueRightFile());
			imgBlueLeft = loadImage(runner.getBlueLeftFile());

			imgBidirectional = loadImage(runner.getBidirectionalFile());

		} catch (IOException e) {
			e.printStackTrace();
		}

		this.setVerticalTextAlignment(Constants.TOP);
		this.setHorizontalPadding(imgRedRight.getWidth());
	}

	/**
	 * Loads the image with the given filename.
	 * 
	 * @param file
	 *            the filename and path of the file to load
	 * @return the loaded image as a BufferedImage
	 * @throws IOException
	 */
	private BufferedImage loadImage(String file) throws IOException {
		java.net.URL imgURL = getClass().getResource(file);
		return ImageIO.read(imgURL);
	}

	/**
	 * Returns the raw shape of the given VisualItem. The raw shape depends on
	 * the adjusted arcWidth and arcHeight.
	 * 
	 * @param item
	 *            the VisualItem of the node
	 * @return the raw shape of the VisualItem
	 */
	protected Shape getRawShape(VisualItem item) {
		Shape shape = super.getRawShape(item);

		if (shape instanceof RoundRectangle2D) {
			Rectangle2D rect = shape.getBounds2D();
			((RoundRectangle2D) shape)
					.setRoundRect(rect.getX(), rect.getY(), rect.getWidth(),
							rect.getHeight() + 10, arcWidth, arcHeight);
		}

		return shape;
	}

	/**
	 * Renders the VisualItem with the Graphics2D and draws the arrow images and
	 * the bars.
	 * 
	 * @param item
	 *            the VisualItem to render
	 * @param g
	 *            the Graphics2D instance for rendering and drawing
	 */
	public void render(Graphics2D g, VisualItem item) {
		super.render(g, item);

		if (barsDisplayed) {
			renderBars(g, item);
		}

		Direction dir = runner.isInVisibleNodesOfClickedItem(item
				.getString(m_labelName));
		VisualItem activeItem = runner.getActiveItem();

		if (dir != null && activeItem != null) {

			if (dir == Direction.In) {
				drawImage(activeItem, item, g, imgRedRight, imgRedLeft);
			} else if (dir == Direction.Out) {
				drawImage(activeItem, item, g, imgBlueLeft, imgBlueRight);
			} else {
				drawImage(activeItem, item, g, imgBidirectional,
						imgBidirectional);
			}

		}
	}

	/**
	 * Draws the bars for visualizing the number of outgoing, incoming and
	 * bidirectional edges.
	 * 
	 * @param g
	 *            the Graphics2D to draw the bars
	 * @param item
	 *            the VisualItem for which the bars are drawn
	 */
	private void renderBars(Graphics2D g, VisualItem item) {
		Rectangle2D rect = getRawShape(item).getBounds2D();

		if (item instanceof NodeItem) {
			NodeItem treeNode = (NodeItem) item;

			DegreeHelper degrees = runner.getDegree(treeNode.getRow());

			if (degrees != null) {

				double in = degrees.getIn();
				double out = degrees.getOut();
				double bi = degrees.getBi();
				double count = runner.getGraph().getNodeCount();

				// subtract the round corners
				double width = defaultWidth;

				int outLength = out == 0 ? 0 : (int) (width * (out / count));
				int inLength = in == 0 ? 0 : (int) (width * (in / count));
				int biLength = bi == 0 ? 0 : (int) (width * (bi / count));

				// start at an indent of arcWidth
				int minX = (int) (rect.getMinX() + imgBlueRight.getWidth());

				int outY = (int) rect.getMaxY() - barDistanceToText
						- barDistance;
				int biY = (int) rect.getMaxY() - barDistance - strokeThickness;
				int inY = (int) rect.getMaxY() - strokeThickness;

				g.setStroke(new BasicStroke(strokeThickness));

				if (outLength != 0) {
					g.setPaint(ColorContext.OutBarColor);
					g.drawLine(minX, outY, minX + outLength, outY);
				}

				if (biLength != 0) {
					g.setPaint(ColorContext.BiBarColor);
					g.drawLine(minX, biY, minX + biLength, biY);
				}

				if (inLength != 0) {
					g.setPaint(ColorContext.InBarColor);
					g.drawLine(minX, inY, minX + inLength, inY);
				}
			}
		}
	}

	/**
	 * Draws the arrow images in the nodes. The drawn arrows are angled so that
	 * they show to the actual focus item
	 * 
	 * @param activeItem
	 *            the selected item in the tree
	 * @param item
	 *            the item in which the image is drawn
	 * @param g
	 *            the Graphics2D instance to draw the image
	 * @param left
	 *            the arrow image which points to the left
	 * @param right
	 *            the arrow image which points to the right
	 */
	private void drawImage(VisualItem activeItem, VisualItem item,
			Graphics2D g, BufferedImage left, BufferedImage right) {

		if (activeItem.getX() >= item.getX()) {
			g.drawImage(left, (int) item.getX()
					+ getRawShape(item).getBounds().width - left.getWidth(),
					(int) item.getY(), left.getWidth(), left.getHeight(),
					new JLabel());
		} else {
			g.drawImage(right, (int) item.getX(), (int) item.getY(), right
					.getWidth(), right.getHeight(), new JLabel());
		}

	}

	/**
	 * Switch the bar display on and off.
	 */
	public void toggle() {
		barsDisplayed = !barsDisplayed;
	}

}
