package treeplus.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.filter.GraphDistanceFilter;
import prefuse.action.layout.graph.ForceDirectedLayout;
import prefuse.activity.Activity;
import prefuse.controls.DragControl;
import prefuse.controls.FocusControl;
import prefuse.controls.PanControl;
import prefuse.controls.WheelZoomControl;
import prefuse.controls.ZoomControl;
import prefuse.controls.ZoomToFitControl;
import prefuse.data.Graph;
import prefuse.data.tuple.TupleSet;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.force.ForceSimulator;
import prefuse.util.ui.JForcePanel;
import prefuse.util.ui.JSearchPanel;
import prefuse.util.ui.JValueSlider;
import prefuse.visual.VisualGraph;
import prefuse.visual.VisualItem;
import treeplus.visualization.demo.ITreePlusRunner;

/**
 * @author Barbara Neubauer, Harald Kstinger, based on Source Code of prefuse
 *         toolkit by <a href="http://jheer.org">jeffrey heer</a>
 */
public class GraphView extends Display {

	private static final String graph = "graph";
	private static final String nodes = "graph.nodes";
	private static final String edges = "graph.edges";
	private static int hops = 30;
	private static ActionList animate;
	private static GraphDistanceFilter filter;

	/**
	 * Creates a GraphView with the given graph, the given node name and the given {@link TreePlusRunner}.
	 * The Visualization is built and the renderers are set. There are also some ActionLists built.
	 * @param g the graph
	 * @param label the node name
	 * @param runner the current {@link TreePlusRunner}
	 */
	public GraphView(Graph g, String label, ITreePlusRunner runner) {
		super(new Visualization());

		// --------------------------------------------------------------------
		// set up the renderers
		LabelRenderer tr = new LabelRenderer();
		tr.setRoundedCorner(8, 8);
		m_vis.setRendererFactory(new DefaultRendererFactory(tr));

		// --------------------------------------------------------------------
		// register the data with a visualization

		// adds graph to visualization and sets renderer label field
		setGraph(g, label);

		// fix selected focus nodes
		TupleSet focusGroup = m_vis.getGroup(Visualization.FOCUS_ITEMS);

		focusGroup
				.addTupleSetListener(new NodeSelectionListener(runner));

		// --------------------------------------------------------------------
		// create actions to process the visual data

		filter = new GraphDistanceFilter(graph, hops);

		ColorAction fill = new ColorAction(nodes, VisualItem.FILLCOLOR,
				ColorLib.rgb(255, 255, 255));
		fill.add(VisualItem.FIXED, ColorLib.rgb(255, 100, 100));
		fill.add(VisualItem.HIGHLIGHT, ColorLib.rgb(255, 200, 125));

		ColorAction search = new ColorAction(Visualization.SEARCH_ITEMS,
				VisualItem.FILLCOLOR, ColorLib.rgb(255, 200, 125));

		ActionList draw = new ActionList();
		draw.add(filter);
		draw.add(fill);
		draw.add(new ColorAction(nodes, VisualItem.STROKECOLOR, ColorLib.rgb(0,
				0, 0)));
		draw.add(new ColorAction(nodes, VisualItem.TEXTCOLOR, ColorLib.rgb(0,
				0, 0)));
		draw.add(new ColorAction(edges, VisualItem.FILLCOLOR, ColorLib
				.gray(200)));
		draw.add(new ColorAction(edges, VisualItem.STROKECOLOR, ColorLib
				.gray(200)));

		animate = new ActionList(Activity.INFINITY);
		animate.add(new ForceDirectedLayout(graph));
		animate.add(fill);
		animate.add(search);
		animate.add(new RepaintAction());

		// finally, we register our ActionList with the Visualization.
		// we can later execute our Actions by invoking a method on our
		// Visualization, using the name we've chosen below.
		m_vis.putAction("draw", draw);
		m_vis.putAction("layout", animate);
		m_vis.runAfter("draw", "layout");

		setForeground(Color.GRAY);
		setBackground(Color.WHITE);
		setSize(700,700);
	    pan(350, 350);

		// main display controls
		addControlListener(new FocusControl(1));
		addControlListener(new DragControl());
		addControlListener(new PanControl());
		addControlListener(new ZoomControl());
		addControlListener(new WheelZoomControl());
		addControlListener(new ZoomToFitControl());

		m_vis.run("draw");

	}

	/**
	 * Sets the graph of the visualization to the given graph and uses the given label for the node names.
	 * @param g the graph
	 * @param label the node name
	 */
	public void setGraph(Graph g, String label) {
		// update labeling
		DefaultRendererFactory drf = (DefaultRendererFactory) m_vis
				.getRendererFactory();
		((LabelRenderer) drf.getDefaultRenderer()).setTextField(label);

		// update graph
		m_vis.removeGroup(graph);
		VisualGraph vg = m_vis.addGraph(graph, g);
		m_vis.setValue(edges, null, VisualItem.INTERACTIVE, Boolean.FALSE);
		VisualItem f = (VisualItem) vg.getNode(0);
		m_vis.getGroup(Visualization.FOCUS_ITEMS).setTuple(f);
		f.setFixed(false);
	}

	// ------------------------------------------------------------------------
	// Main and demo methods
	/**
	 * This method returns a Panel including a GraphView graph and a
	 * panel with controls for manipulation the graph visualization.
	 * @param g the graph
	 * @param label the node name
	 * @param runner the current {@link TreePlusRunner}
	 * @return the GraphView JComponent
	 */
	public static JComponent demo(Graph g, String label, ITreePlusRunner runner) {
		final GraphView view = new GraphView(g, label, runner);
		JPanel panel = new JPanel(new BorderLayout());

		// create a search panel for the tree map
		JSearchPanel search = new JSearchPanel(view.getVisualization(), nodes,
				Visualization.SEARCH_ITEMS, label, true, true);
		search.setShowResultCount(true);
		search.setBorder(BorderFactory.createEmptyBorder(5, 5, 4, 0));
		search.setFont(FontLib.getFont("Tahoma", Font.PLAIN, 11));
		search.setBackground(panel.getBackground());
		Box box = new Box(BoxLayout.X_AXIS);
		box.add(Box.createHorizontalGlue());
		box.add(search);
		box.add(Box.createHorizontalStrut(3));

		// create a panel for editing force values
		ForceSimulator fsim = ((ForceDirectedLayout) animate.get(0))
				.getForceSimulator();		
		JForcePanel fpanel = new JForcePanel(fsim);

		final JValueSlider slider = new JValueSlider("Distance", 0, hops, hops);
		slider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				filter.setDistance(slider.getValue().intValue());
				view.getVisualization().run("draw");
			}
		});
		
		slider.setBackground(Color.WHITE);
		slider.setPreferredSize(new Dimension(300, 30));
		slider.setMaximumSize(new Dimension(300, 30));

		Box cf = new Box(BoxLayout.Y_AXIS);
		cf.add(slider);
		cf.setBorder(BorderFactory.createTitledBorder("Connectivity Filter"));
		fpanel.add(cf);

		fpanel.add(Box.createVerticalGlue());

		// create a new JSplitPane to present the interface
		JSplitPane split = new JSplitPane();
		split.setLeftComponent(view);
		split.setRightComponent(fpanel);
		split.setOneTouchExpandable(true);
		//split.setContinuousLayout(false);
		split.setDividerLocation(700);
		view.setBorder(new LineBorder(Color.black));
		panel.add(split, BorderLayout.CENTER);
		panel.add(box, BorderLayout.SOUTH);
		return panel;
	}
}
