package treeplus.visualization;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import prefuse.data.Graph;
import prefuse.data.Table;
import prefuse.util.io.IOLib;
import treeplus.visualization.demo.ITreePlusRunner;

/**
 * {@link AbstractAction} that loads a graph file and let the user select the name for the nodes.
 * @author Neubauer, Kstinger
 *
 */
public class OpenGraphAction extends AbstractAction {
    private ITreePlusRunner runner;

    /**
     * Creates an OpenGraphAction with given {@link TreePlusRunner} and
     * sets the AbstractAction.NAME and AbstractAction.ACCELERATOR_KEY.
     * @param runner the current {@link TreePlusRunner}
     */
    public OpenGraphAction(ITreePlusRunner runner) {
        this.runner = runner;
        this.putValue(AbstractAction.NAME, "Open File...");
        this.putValue(AbstractAction.ACCELERATOR_KEY,
                      KeyStroke.getKeyStroke("ctrl O"));
    }
    
    /**
     * Loads the graph file and set the new graph in the TreePlusRunner.
     * @param e the ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
    	        
        Graph g = IOLib.getGraphFile(runner.getEnclosingFrame());
        
        if ( g == null ) 
        	return;
        
        String label = getLabel(runner, g);
        
        if ( label != null ) {
            runner.setGraph(g, label);
            runner.setActualGraphView(null);
        }
        
    }
    
    /**
     * Opens a JDialog to let the user choose the name for the nodes in the graph and returns this name.
     * @param runner the current {@link TreePlusRunner}
     * @param g the graph
     * @return the name for the nodes
     */
    public static String getLabel(ITreePlusRunner runner, Graph g) {
        // get the column names
        Table t = g.getNodeTable();
        int  cc = t.getColumnCount();
        String[] names = new String[cc];
        for ( int i=0; i<cc; ++i )
            names[i] = t.getColumnName(i);
        
        // where to store the result
        final String[] label = new String[1];

        // -- build the dialog -----
        // we need to get the enclosing frame first
        JFrame enclosingFrame = runner.getEnclosingFrame();
        
        final JDialog dialog = new JDialog(
                enclosingFrame, "Choose Label Field", true);
        
        // create the ok/cancel buttons
        final JButton ok = new JButton("OK");
        ok.setEnabled(false);
        ok.addActionListener(new ActionListener() {
           public void actionPerformed(ActionEvent e) {
               dialog.setVisible(false);
           }
        });
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                label[0] = null;
                dialog.setVisible(false);
            }
        });
        
        // build the selection list
        final JList list = new JList(names);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.getSelectionModel().addListSelectionListener(
        new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                int sel = list.getSelectedIndex(); 
                if ( sel >= 0 ) {
                    ok.setEnabled(true);
                    label[0] = (String)list.getModel().getElementAt(sel);
                } else {
                    ok.setEnabled(false);
                    label[0] = null;
                }
            }
        });
        JScrollPane scrollList = new JScrollPane(list);
        
        JLabel title = new JLabel("Choose a field to use for node labels:");
        
        // layout the buttons
        Box bbox = new Box(BoxLayout.X_AXIS);
        bbox.add(Box.createHorizontalStrut(5));
        bbox.add(Box.createHorizontalGlue());
        bbox.add(ok);
        bbox.add(Box.createHorizontalStrut(5));
        bbox.add(cancel);
        bbox.add(Box.createHorizontalStrut(5));
        
        // put everything into a panel
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(title, BorderLayout.NORTH);
        panel.add(scrollList, BorderLayout.CENTER);
        panel.add(bbox, BorderLayout.SOUTH);
        panel.setBorder(BorderFactory.createEmptyBorder(5,2,2,2));
        
        // show the dialog
        dialog.setContentPane(panel);
        dialog.pack();
        dialog.setLocationRelativeTo(enclosingFrame);
        dialog.setVisible(true);
        dialog.dispose();
        
        // return the label field selection
        return label[0];
    }
}