package treeplus.visualization;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import prefuse.Visualization;
import prefuse.activity.ActivityManager;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import treeplus.visualization.demo.ITreePlusRunner;

/**
 * helper class for the preview panel action to display the nodes visible and invisible to the current
 * selected one
 * @author Neubauer, Kstinger
 *
 */
public class PreviewAdjacentNodesAction  {
	
	private ITreePlusRunner runner;
	private String label;
	private JPanel previewPanel;
	
	/**
	 * creates a new action for the preview panel to display current visible
	 * and invisible nodes adjacent to the current selected node.
	 * @param label the label to infer the name of the nodes
	 * @param previewPanel the panel to display the nodes at
	 * @param runner the runner instance
	 */
	public PreviewAdjacentNodesAction(String label,
			JPanel previewPanel, ITreePlusRunner runner) {
		this.label = label;
		this.runner = runner;
		this.previewPanel = previewPanel;
	}
		
	/**
	 * This method is called when the item is clicked. In that case the adjacent
	 * nodes of the item are calculated and visualized in the preview panel.
	 * 
	 * @param item
	 *            the VisualItem which was clicked
	 */
	public void runPreviewAdjacentNodesAction(VisualItem item){
				
		Visualization vis = item.getVisualization();
		
		if ( item instanceof NodeItem) {
			NodeItem treeNode = (NodeItem) item;
			Graph g = runner.getGraph();

			if (item.getRow() >= runner.getGraph().getNodeCount())
				return;

			Node graphNode = g.getNode(treeNode.getRow());

			previewPanel.removeAll();

			Iterator<Edge> inEdges = g.inEdges(graphNode);
			List<Node> in = TreePlusGraphHelper.findAdjacentNodes(inEdges,
					graphNode);

			Iterator<Edge> outEdges = g.outEdges(graphNode);
			List<Node> out = TreePlusGraphHelper.findAdjacentNodes(
					outEdges, graphNode);
			
			processAdjacentNodes(in, out, treeNode.getGraph(), graphNode, vis);
			
			previewPanel.revalidate();
			previewPanel.updateUI();
		}
	}
	/**
	 * This method processes the given adjacent nodes depending on their direction.
	 * The whole information is put into a JScrollPane and is added to the preview panel.
	 * @param in the incoming adjacent nodes
	 * @param out the outgoing adjacent nodes
	 * @param graph the graph
	 * @param selectedNode the actual selected node
	 */
	private void processAdjacentNodes(List<Node> in, List<Node> out,
			Graph graph, Node selectedNode, Visualization vis) {

		Dictionary<Node, Direction> visibleNodes = new Hashtable<Node, Direction>();
		runner.setAdjacentNodesOfSelectedItem(visibleNodes);
		Dictionary<Node, Direction> invisibleNodes = new Hashtable<Node, Direction>();
		
		List<Integer> visibleItems = getVisibleItems(vis);

		processNodes(in, Direction.In, visibleNodes, invisibleNodes, graph, selectedNode, visibleItems);
		processNodes(out, Direction.Out, visibleNodes, invisibleNodes, graph, selectedNode, visibleItems);

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		JScrollPane listScroller = new JScrollPane(panel);
		listScroller.setPreferredSize(new Dimension(
				previewPanel.getWidth() - 10, previewPanel.getHeight()));

		renderNodes(visibleNodes, "visible Adjacent nodes", panel);
		renderNodes(invisibleNodes, "invisible Adjacent nodes", panel);
		
		previewPanel.add(listScroller);
	}


	/**
	 * returns the current visible items (Nodes) on the display. 
	 * We just need to know about the nodes 'cause they are used to infer the visible and invisible list
	 * of adjacent nodes
	 * @param vis the visualization 
	 * @return a list of visible nodes (their row index in the table)
	 */
	private List<Integer> getVisibleItems(Visualization vis) {
		List<Integer> visibleItems = new ArrayList<Integer>();
		
		Iterator<VisualItem> visIt = vis.visibleItems();
		
		while (visIt.hasNext()) {
			VisualItem item = visIt.next();
			
			if (item instanceof Node)			
				visibleItems.add(item.getRow());
			
			if (item instanceof Edge)
			{
				Edge edge = (Edge)item;				
				visibleItems.add(edge.getInt("source"));
				visibleItems.add(edge.getInt("target"));
			}		
		}
		return visibleItems;
	}

	/**
	 * This method adds the information about the given nodes to JLabels and put them together
	 * into a Panel. 
	 * @param nodes the list of nodes to render
	 * @param header the header for the list, defines if the nodes are visible or not
	 * @param panel the panel to add labels with the information about the nodes
	 */
	private void renderNodes(Dictionary<Node, Direction> nodes, String header,
			JPanel panel) {
		
		// avoid a single label with empty nodes
		if (nodes == null || nodes.isEmpty())
			return;

		JLabel heading = new JLabel(header, JLabel.CENTER);
		heading.setFont(new Font("Tahoma", Font.BOLD, 11));
		heading.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		panel.add(heading);
		panel.add(Box.createRigidArea(new Dimension(0, 10)));
		
		for (Enumeration<Node> keys = nodes.keys(); keys.hasMoreElements();) {
			Node node = keys.nextElement();
			Direction direction = nodes.get(node);
			JLabel nodeLabel = null;
			
			if (direction == Direction.In) {
				nodeLabel = getNodeLabel(node.getString(label), runner
						.getRedRightFile(), ColorContext.NodeInColor);
			} else if (direction == Direction.Out ){
				nodeLabel = getNodeLabel(node.getString(label), runner
						.getBlueLeftFile(), ColorContext.NodeOutColor);
			} else {
				nodeLabel = getNodeLabel(node.getString(label), runner
						.getBidirectionalFile(), ColorContext.NodeBiColor);
			}

			panel.add(nodeLabel);
			panel.add(Box.createRigidArea(new Dimension(0, 5)));
		}

	}

	/**
	 * Returns a JLabel with the given properties.
	 * @param name the node name
	 * @param imageFile the arrow file name
	 * @param colorAsInt the background color for the label
	 * @return the JLabel with the given properties
	 */
	private JLabel getNodeLabel(String name, String imageFile, int colorAsInt) {

		java.net.URL imgURL = getClass().getResource(imageFile);
		ImageIcon icon = new ImageIcon(imgURL);

		JLabel nodeLabel = new JLabel(name, icon, JLabel.LEADING);
		nodeLabel.setOpaque(true);

		Color color = new Color(colorAsInt);
		nodeLabel.setBackground(color);

		return nodeLabel;
	}

	/**
	 * Runs through the given list of adjacent nodes and checks for every node if it's currently
	 * visible in the tree. Depending on that the node is put into the visible or invisible list of nodes.
	 * @param list the list of adjacent nodes
	 * @param direction the direction of the nodes in the list (Direction.IN, Direction.OUT)
	 * @param visibleNodes a Dictionary which saves the currently visible nodes
	 * @param invisibleNodes a Dictionary which saves the currently invisible nodes
	 * @param graph the graph
	 * @param selectedNode the actual selected node in the tree
	 */
	private void processNodes(List<Node> list, Direction direction,
			Dictionary<Node, Direction> visibleNodes,
			Dictionary<Node, Direction> invisibleNodes, Graph graph, Node selectedNode, List<Integer> visibleItems) {

		boolean visible = true;
		Direction dir = direction;
		
		for (Node node : list) {

			visible = visibleItems.contains(node.getRow());

			if (TreePlusGraphHelper.isBidirectionalAdjacent(runner.getGraph(), 
					selectedNode.getRow(), node.getRow()))
				dir = Direction.Bi;
			
			if (visible)
				visibleNodes.put(node, dir);
			else
				invisibleNodes.put(node, dir);
			
			dir = direction;
		}
	}
}

