package treeplus.visualization;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Table;
import prefuse.data.Tree;
import prefuse.data.Tuple;
import prefuse.data.tuple.TupleManager;

/**
 * New spanning tree based on existing implementation of spanning tree of 
 * the prefuse toolkit. This class uses a given graph to determine a new spanning tree out of it 
 * according to the given label (description field of the node). 
 * 
 * @author Neubauer, Kstinger
 */
public class SpanningTreePlus extends Tree {

	/**
     * Create a new SpanningTreePlus out of the given graph, starting from the node having the 
     * most adjacent other nodes.
     * 
     * @param g the backing Graph to span
     * @param label the field in the node to determine the description
     */
	public SpanningTreePlus(Graph g, String label) {
		this(g, null, label);
	}
	
    /**
     * Create a new SpanningTreePlus out of the given graph, starting from the given node.
     * 
     * @param g the backing Graph to span
     * @param node the Node to use as the root of the spanning tree
     * @param label the field in the node to determine the description
     */
    public SpanningTreePlus(Graph g, Node node, String label) {
    	    	
    	Tree t = null;
        
    	// first build the spanning tree from the graph
        try {            
        	if (node != null)
        		t = g.getSpanningTree(node);
        	else 
        		t = g.getSpanningTree();
        	
        } catch ( Exception e ) {
            e.printStackTrace();
            System.exit(1);
        }   

        // the standard spanning tree has different column naming
        // to fix this "bug", copy them all to a new table, add a new column for the
        // label and work with this new table instead
        
        Table nodes = t.getNodeTable();
        Table edges = t.getEdgeTable();
        
        Table newTreeNodes = getNodeTable();
        newTreeNodes.addColumn(label, String.class);
        
        for(int i=0;i<nodes.getRowCount();i++){
        	int row = newTreeNodes.addRow();
        	newTreeNodes.setString(row, label, nodes.getString(i,label));
        }       
        
        Table newTreeEdges = getEdgeTable();
        
        for(int i=0;i<edges.getRowCount();i++){
        	int row = newTreeEdges.addRow(); 
        	
        	newTreeEdges.set(row, "source", edges.get(i,"parent"));	
        	newTreeEdges.set(row, "target", edges.get(i,"child"));        	
        }       
        
    }
        
    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#addChild(int)
     */
    public int addChild(int parent) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#addChild(prefuse.data.Node)
     */
    public Node addChild(Node parent) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#addChildEdge(int, int)
     */
    public int addChildEdge(int parent, int child) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#addChildEdge(prefuse.data.Node, prefuse.data.Node)
     */
    public Edge addChildEdge(Node parent, Node child) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#removeChild(int)
     */
    public boolean removeChild(int node) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#removeChild(prefuse.data.Node)
     */
    public boolean removeChild(Node n) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#removeChildEdge(prefuse.data.Edge)
     */
    public boolean removeChildEdge(Edge e) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Tree#removeChildEdge(int)
     */
    public boolean removeChildEdge(int edge) {
        throw new UnsupportedOperationException(
            "Changes to tree structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.tuple.TupleSet#clear()
     */
    public void clear() {
        throw new UnsupportedOperationException(
            "Changes to graph structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.tuple.TupleSet#removeTuple(prefuse.data.Tuple)
     */
    public boolean removeTuple(Tuple t) {
        throw new UnsupportedOperationException(
            "Changes to graph structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Graph#setEdgeTable(prefuse.data.Table)
     */
    public void setEdgeTable(Table edges) {
        throw new UnsupportedOperationException(
            "Changes to graph structure not allowed for spanning trees.");
    }

    /**
     * Unsupported operation. Spanning trees should not be edited.
     * @see prefuse.data.Graph#setTupleManagers(prefuse.data.tuple.TupleManager, prefuse.data.tuple.TupleManager)
     */
    public void setTupleManagers(TupleManager ntm, TupleManager etm) {
        throw new UnsupportedOperationException(
            "Changes to graph structure not allowed for spanning trees.");
    }    
} 
