package treeplus.visualization;

import java.awt.event.MouseEvent;

import prefuse.controls.ControlAdapter;
import prefuse.data.Node;
import prefuse.util.ui.JFastLabel;
import prefuse.visual.VisualItem;
import treeplus.visualization.demo.ITreePlusRunner;
import treeplus.visualization.demo.TreePlusRunner;

/**
 * {@link ControlAdapter} to catch the events of entering, exiting and clicking
 * a VisualItem. Depending on the event different actions are started. By
 * entering an item the tooltip for the item is shown. By clicking an item the
 * adjacent nodes of this item are caught and shown in the preview panel.
 * 
 * @author Neubauer, Kstinger
 * 
 */
public class TreeNodeControlAdapter extends ControlAdapter {

	private ITreePlusRunner runner;
	private JFastLabel title;
	private String label;
	private PreviewAdjacentNodesAction action;

	/**
	 * Creates a TreeNodeControlAdapter with the label name of the nodes, the
	 * JFastLabel for the tooltip, the preview Panel for visualizing adjacent
	 * nodes and the current {@link TreePlusRunner}.
	 * 
	 * @param label
	 *            the node name
	 * @param title
	 *            the JFastLabel for showing the tooltip with the number of
	 *            outgoing, incoming and bidirectional adjacent nodes
	 * @param runner
	 *            the current {@link TreePlusRunner} for getting and setting
	 *            information about the visualiziation
	 */
	public TreeNodeControlAdapter(String label, JFastLabel title,
			ITreePlusRunner runner, PreviewAdjacentNodesAction action) {
		this.label = label;
		this.title = title;
		this.runner = runner;
		this.action = action;
	}

	/**
	 * This method is called when a VisualItem is entered. The tooltip is set
	 * for the item and put into the tooltip label. If the item is a dummy node
	 * which visualizes the number of moved nodes the list of moved nodes for
	 * this item is set.
	 * 
	 * @param item
	 *            the VisualItem which was entered
	 * @param e
	 *            the MouseEvent
	 */
	public void itemEntered(VisualItem item, MouseEvent e) {

		if (item instanceof Node) {

			int row = item.getRow();
			int source = runner.getSourceNodeForDummyNode(row);

			if (source != -1) {
				runner.setMovedListOfItem(runner
						.getMovedNodesForSourceNode(source));
			} else
				runner.setMovedListOfItem(null);

			if (item.canGetString(label)) {
				title.setText(runner.getToolTipForNodeRow(item.getRow()));
			}

			item.getVisualization().run("repaint");

		}
	}

	/**
	 * Method is called when the item is exited. The tooltip Label text and the
	 * list of moved nodes is set to null.
	 * 
	 * @param item
	 *            the VisualItem which was exited
	 * @param e
	 *            the MouseEvent
	 */
	public void itemExited(VisualItem item, MouseEvent e) {
		title.setText(null);
		runner.setMovedListOfItem(null);
		item.getVisualization().run("repaint");
	}

	/**
	 * This method is called when the item is clicked. In that case the adjacent
	 * nodes of the item are calculated and visualized in the preview panel.
	 * 
	 * @param item
	 *            the VisualItem which was clicked
	 * @param mouse
	 *            the MouseEvent
	 */
	public void itemClicked(VisualItem item, MouseEvent mouse) {

		if (mouse.getClickCount() == 1
				&& mouse.getButton() == MouseEvent.BUTTON1) {
			runner.setActiveItem(item);
			action.runPreviewAdjacentNodesAction(item);
		}

	}

}
