package treeplus.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.geom.Point2D;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.ItemAction;
import prefuse.action.RepaintAction;
import prefuse.action.animate.ColorAnimator;
import prefuse.action.animate.LocationAnimator;
import prefuse.action.animate.QualityControlAnimator;
import prefuse.action.animate.VisibilityAnimator;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.FontAction;
import prefuse.action.filter.FisheyeTreeFilter;
import prefuse.action.layout.CollapsedSubtreeLayout;
import prefuse.action.layout.graph.NodeLinkTreeLayout;
import prefuse.activity.SlowInSlowOutPacer;
import prefuse.controls.PanControl;
import prefuse.controls.WheelZoomControl;
import prefuse.controls.ZoomControl;
import prefuse.controls.ZoomToFitControl;
import prefuse.data.Tree;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.search.PrefixSearchTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.render.AbstractShapeRenderer;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.ui.JFastLabel;
import prefuse.util.ui.JSearchPanel;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;

import treeplus.visualization.demo.ITreePlusRunner;

/**
 *  {@link Display} for the TreePlus visualization. TreePlus is based on Tree data and adds the Renderers and 
 *  Listeners to the visualization. With the ActionLists the visualization can be changed and updated.
 * @author Neubauer, Kstinger
 *
 */
public class TreePlus extends Display {

	private static final String tree = "tree";
	private static final String treeNodes = "tree.nodes";
	private static final String treeEdges = "tree.edges";

	private LabelRenderer m_nodeRenderer;
	private EdgeRenderer m_edgeRenderer;

	private int m_orientation = Constants.ORIENT_LEFT_RIGHT;

	private Tree shownTree = null;
	private DefaultRendererFactory renderFactory;

	/**
	 * Creates a new TreePlus instance with the given Tree, the given name of the nodes
	 * and the given {@link TreePlusRunner}.
	 * The given tree is put into the visualization and Renderers and Listeners are added.
	 * @param t the Tree which forms the basis for the TreePlus-Tree
	 * @param runner the current {@link TreePlusRunner}
	 * @param label the name of the nodes
	 */
	public TreePlus(Tree t, ITreePlusRunner runner, String label, PreviewAdjacentNodesAction adjacentNodesAction) {
		super(new Visualization());

		shownTree = t;

		m_vis.add(tree, t);

		CreateBarLabelRenderer(runner);

		m_edgeRenderer = new EdgeRenderer(Constants.EDGE_TYPE_CURVE,
				Constants.EDGE_ARROW_FORWARD);

		renderFactory = new DefaultRendererFactory(m_nodeRenderer);
		renderFactory.add(new InGroupPredicate(treeEdges), m_edgeRenderer);
		m_vis.setRendererFactory(renderFactory);

		// colors
		ItemAction nodeColor = new NodeColorAction(treeNodes, runner);
		ItemAction textColor = new ColorAction(treeNodes, VisualItem.TEXTCOLOR,
				ColorLib.rgb(0, 0, 0));
		m_vis.putAction("textColor", textColor);

		ItemAction edgeColor = new EdgeColorAction(treeEdges);

		// quick repaint
		ActionList repaint = new ActionList();
		repaint.add(nodeColor);
		repaint.add(new RepaintAction());
		m_vis.putAction("repaint", repaint);

		// full paint
		ActionList fullPaint = new ActionList();
		fullPaint.add(nodeColor);
		m_vis.putAction("fullPaint", fullPaint);

		// animate paint change
		ActionList animatePaint = new ActionList(400);
		animatePaint.add(new ColorAnimator(treeNodes));
		animatePaint.add(new RepaintAction());
		m_vis.putAction("animatePaint", animatePaint);

		// create the tree layout action
		NodeLinkTreeLayout treeLayout = new NodeLinkTreeLayout(tree,
				m_orientation, 50, 0, 8);
		treeLayout.setLayoutAnchor(new Point2D.Double(25, 300));
		m_vis.putAction("treeLayout", treeLayout);

		CollapsedSubtreeLayout subLayout = new CollapsedSubtreeLayout(tree,
				m_orientation);
		m_vis.putAction("subLayout", subLayout);

		AutoPanAction autoPan = new AutoPanAction(this);

		FisheyeTreeFilter fisheye = new FisheyeTreeFilter(tree, 0);

		// create the filtering and layout
		ActionList filter = new ActionList();
		filter.add(fisheye);
		filter.add(new FontAction(treeNodes, FontLib.getFont("Tahoma", 16)));
		filter.add(treeLayout);
		filter.add(subLayout);
		filter.add(textColor);
		filter.add(nodeColor);
		filter.add(edgeColor);
		m_vis.putAction("filter", filter);

		// animated transition
		ActionList animate = new ActionList(1000);
		animate.setPacingFunction(new SlowInSlowOutPacer());
		animate.add(autoPan);
		animate.add(new QualityControlAnimator());
		animate.add(new VisibilityAnimator(tree));
		animate.add(new LocationAnimator(treeNodes));
		animate.add(new ColorAnimator(treeNodes));
		animate.add(new RepaintAction());
		m_vis.putAction("animate", animate);
		m_vis.alwaysRunAfter("filter", "animate");
		
		// initialize the display
		setSize(700, 600);
		PopupMenuControler popup = new PopupMenuControler(runner);

		addControlListener(new ZoomToFitControl());
		addControlListener(new ZoomControl());
		addControlListener(new WheelZoomControl());
		addControlListener(new PanControl());
		addControlListener(new TreePlusToolTipControl(runner));
		addControlListener(new TreePlusFocusControl(2, "filter", runner,
				shownTree, fisheye, adjacentNodesAction));
		addControlListener(popup);

		// ------------------------------------------------

		// filter graph and perform layout
		setOrientation(Constants.ORIENT_LEFT_RIGHT);
		m_vis.run("filter");

		TupleSet search = new PrefixSearchTupleSet();
		m_vis.addFocusGroup(Visualization.SEARCH_ITEMS, search);
		search.addTupleSetListener(new TupleSetListener() {
			public void tupleSetChanged(TupleSet t, Tuple[] add, Tuple[] rem) {
				m_vis.cancel("animatePaint");
				m_vis.run("fullPaint");
				m_vis.run("animatePaint");
			}
		});

	}
	/**
	 * Creates a new BarLabelRenderer for the nodes of the tree.
	 * @param the current {@link TreePlusRunner} 
	 */
	private void CreateBarLabelRenderer(ITreePlusRunner runner) {
		m_nodeRenderer = new BarLabelRenderer(runner);
		m_nodeRenderer
				.setRenderType(AbstractShapeRenderer.RENDER_TYPE_DRAW_AND_FILL);
		m_nodeRenderer.setRoundedCorner(8, 8);
	}

	/**
	 * Sets the orientation of the visualization of the tree.
	 * @param orientation (Constants.ORIENT_LEFT_RIGHT, Constants.ORIENT_RIGHT_LEFT
	 * 					Constants.ORIENT_TOP_BOTTOM,Constants.ORIENT_BOTTOM_TOP)
	 */
	public void setOrientation(int orientation) {

		NodeLinkTreeLayout rtl = (NodeLinkTreeLayout) m_vis
				.getAction("treeLayout");
		CollapsedSubtreeLayout stl = (CollapsedSubtreeLayout) m_vis
				.getAction("subLayout");

		switch (orientation) {
		case Constants.ORIENT_LEFT_RIGHT:
			m_nodeRenderer.setHorizontalAlignment(Constants.LEFT);
			m_edgeRenderer.setHorizontalAlignment1(Constants.RIGHT);
			m_edgeRenderer.setHorizontalAlignment2(Constants.LEFT);
			m_edgeRenderer.setVerticalAlignment1(Constants.CENTER);
			m_edgeRenderer.setVerticalAlignment2(Constants.CENTER);
			break;
		case Constants.ORIENT_RIGHT_LEFT:
			m_nodeRenderer.setHorizontalAlignment(Constants.RIGHT);
			m_edgeRenderer.setHorizontalAlignment1(Constants.LEFT);
			m_edgeRenderer.setHorizontalAlignment2(Constants.RIGHT);
			m_edgeRenderer.setVerticalAlignment1(Constants.CENTER);
			m_edgeRenderer.setVerticalAlignment2(Constants.CENTER);
			break;
		case Constants.ORIENT_TOP_BOTTOM:
			m_nodeRenderer.setHorizontalAlignment(Constants.CENTER);
			m_edgeRenderer.setHorizontalAlignment1(Constants.CENTER);
			m_edgeRenderer.setHorizontalAlignment2(Constants.CENTER);
			m_edgeRenderer.setVerticalAlignment1(Constants.BOTTOM);
			m_edgeRenderer.setVerticalAlignment2(Constants.TOP);
			break;
		case Constants.ORIENT_BOTTOM_TOP:
			m_nodeRenderer.setHorizontalAlignment(Constants.CENTER);
			m_edgeRenderer.setHorizontalAlignment1(Constants.CENTER);
			m_edgeRenderer.setHorizontalAlignment2(Constants.CENTER);
			m_edgeRenderer.setVerticalAlignment1(Constants.TOP);
			m_edgeRenderer.setVerticalAlignment2(Constants.BOTTOM);
			break;
		default:
			throw new IllegalArgumentException(
					"Unrecognized orientation value: " + orientation);
		}

		m_orientation = orientation;
		rtl.setOrientation(orientation);
		stl.setOrientation(orientation);
	}

	/**
	 * Returns the actual orientation of the visualization.
	 * @return the orientation
	 */
	public int getOrientation() {
		return m_orientation;
	}

	/**
	 * Calls the toggle method in the BarLabelRenderer for switching on and off the bars.
	 * 
	 */
	public void toggleRenderer() {
		BarLabelRenderer renderer = (BarLabelRenderer) m_nodeRenderer;
		renderer.toggle();
		m_vis.run("filter");
	}

	/**
	 * Creates and returns a JPanel including the TreePlus tree, a search Label, 
	 * a Label for the tooltip and the Buttons to activate/deactivate legend and hints.
	 * @param tree the tree for building the TreePlus tree
	 * @param label the node name
	 * @param runner the current {@link TreePlusRunner}
	 * @return the Panel with all TreePlus components
	 */
	public static JComponent demo(Tree tree, final String label,
			ITreePlusRunner runner) {

		Color BACKGROUND = Color.WHITE;
		Color FOREGROUND = Color.BLACK;
		
		final JPanel previewPanel = new JPanel();
		previewPanel.setPreferredSize(new Dimension(200, 0));
		previewPanel.add(new JLabel("Preview"));

		PreviewAdjacentNodesAction adjacentNodesAction = new PreviewAdjacentNodesAction(label, previewPanel, runner);
		
		// create a new treeplus
		final TreePlus tview = new TreePlus(tree, runner, label, adjacentNodesAction);
		tview.setBackground(BACKGROUND);
		tview.setForeground(FOREGROUND);
		
		final JFastLabel titleLabel = new JFastLabel("                 ");
		titleLabel.setPreferredSize(new Dimension(350, 20));
		titleLabel.setVerticalAlignment(SwingConstants.BOTTOM);
		titleLabel.setBorder(BorderFactory.createEmptyBorder(3, 0, 0, 0));
		titleLabel.setFont(FontLib.getFont("Tahoma", Font.PLAIN, 11));

		// create a search panel for the tree map
		JSearchPanel search = new JSearchPanel(tview.getVisualization(),
				treeNodes, Visualization.SEARCH_ITEMS, label, true, true);
		search.setShowResultCount(true);
		search.setBorder(BorderFactory.createEmptyBorder(5, 5, 4, 0));
		search.setFont(FontLib.getFont("Tahoma", Font.PLAIN, 11));
		search.setBackground(previewPanel.getBackground());

		tview.addControlListener(new TreeNodeControlAdapter(label, titleLabel,
				runner, adjacentNodesAction));

		JPanel legendPanel = createLegendPanel(runner);

		Box box = new Box(BoxLayout.X_AXIS);
		JButton legend = new JButton(new LegendAction(legendPanel));
		legend.setText("Show legend");
		box.add(legend);
		box.add(Box.createHorizontalStrut(10));
		box.add(new JButton(new ToggleHintsAction(tview)));
		box.add(Box.createHorizontalStrut(10));
		box.add(titleLabel);
		box.add(Box.createHorizontalGlue());
		box.add(search);
		box.add(Box.createHorizontalStrut(3));

		JPanel centerPanel = new JPanel(new GridBagLayout());
		GridBagConstraints g = new GridBagConstraints();
		g.gridx = 0;
		g.gridy = 0;
		g.weightx = 0.7;
		g.weighty = 1.0;
		g.fill = GridBagConstraints.BOTH;
		centerPanel.add(tview, g);

		g.weightx = 0.3;
		g.gridx = 1;
		centerPanel.add(previewPanel, g);

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(tview, BorderLayout.CENTER);
		panel.add(box, BorderLayout.SOUTH);
		panel.add(previewPanel, BorderLayout.EAST);
		panel.add(legendPanel, BorderLayout.NORTH);
		return panel;
	}

	/**
	 * Returns a Panel with the legend for the TreePlus visualization.
	 * @param runner the current {@link TreePlusRunner}
	 * @return the legend Panel
	 */
	private static JPanel createLegendPanel(ITreePlusRunner runner) {
		// creating legend
		JPanel legendPanel = new JPanel();
		legendPanel.setLayout(new BoxLayout(legendPanel, BoxLayout.X_AXIS));
		JLabel blueLegend = new JLabel("eaten by the focus", runner
				.getLegendIconBlue(), JLabel.LEADING);
		legendPanel.add(blueLegend);
		legendPanel.add(Box.createHorizontalStrut(15));
		JLabel biLegend = new JLabel("eat or eaten by the focus", runner
				.getLegendIconBi(), JLabel.LEADING);
		legendPanel.add(biLegend);
		legendPanel.add(Box.createHorizontalStrut(15));
		JLabel redLegend = new JLabel("eat the focus", runner
				.getLegendIconRed(), JLabel.LEADING);
		legendPanel.add(redLegend);
		legendPanel.add(Box.createHorizontalStrut(15));
		JLabel activeLegend = new JLabel("focus node", runner
				.getLegendIconActive(), JLabel.LEADING);
		legendPanel.add(activeLegend);
		legendPanel.add(Box.createHorizontalStrut(15));
		JLabel searchLegend = new JLabel("search result", runner
				.getLegendIconSearch(), JLabel.LEADING);
		legendPanel.add(searchLegend);
		legendPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		legendPanel.setVisible(false);
		return legendPanel;
	}
	
	/**
	 * Returns the depth of the actual visible tree
	 * @return the depth of the tree
	 */
	public int getTreeDepth() {
		return shownTree.getDepth(shownTree.getRoot().getRow());
	}
}
