package treeplus.visualization;

import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import prefuse.Visualization;
import prefuse.action.filter.FisheyeTreeFilter;
import prefuse.controls.FocusControl;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Tree;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ui.UILib;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import treeplus.visualization.demo.ITreePlusRunner;

/**
 * {@link FocusControl} that handles the focus event on a node at a given amount
 * of clicks. Moreover, this class performs actions on the underlying tree to handle moved nodes
 * from subtree to subtree by inserting dummy-nodes showing information about how many nodes moved from 
 * which subtree to which other subtree.
 * 
 * @author Neubauer, Kstinger
 * 
 */
public class TreePlusFocusControl extends FocusControl {

	private String group = Visualization.FOCUS_ITEMS;
	private ITreePlusRunner runner;
	private Tree tree;
	private Hashtable<Integer, Integer> movedNodesCountPerNode = null;
	private Hashtable<Integer, Integer> movedNodeIdsPerNode = null;
	private FisheyeTreeFilter fisheye;
	private PreviewAdjacentNodesAction action;	
	
	/**
	 * Creates the focus control
	 * @param clicks the amount of clicks which have to occur to execute this control
	 * @param act the action which has to be performed after the clicks occurred
	 * @param runner the runner to determine the underlying graph and various other variables
	 * @param tree the currently shown and rendered tree
	 * @param fisheye the fisheye filter which is used to determine, how many levels of the tree stay open
	 */
	public TreePlusFocusControl(int clicks, String act, ITreePlusRunner runner,
			Tree tree, FisheyeTreeFilter fisheye, PreviewAdjacentNodesAction action) {
		super(clicks, act);

		this.runner = runner;
		this.tree = tree;

		movedNodesCountPerNode = new Hashtable<Integer, Integer>();
		movedNodeIdsPerNode = new Hashtable<Integer, Integer>();

		this.fisheye = fisheye;
		this.action  = action;
	}

	/**
	 * Executed, when an item is clicked. The item is checked if it belongs to the
	 * current filter attributes and if the amount of clicks is met.
	 * @param item the clicked visual item
	 * @param e the mouse event
	 */
	@Override
	public void itemClicked(VisualItem item, MouseEvent e) {

		if (!filterCheck(item))
			return;

		if (UILib.isButtonPressed(e, button) && e.getClickCount() == ccount) {

			// call the method to build the correct subtree for the current selected item
			// from the graph. this means, that all the corresponding edges from the original graph
			// are shown as child nodes, except the root of this subtree (which of course cannot change
			// the position during rebuilding -> the tree would rebuild)
			buildSubtreeFromGraph(item);

			Visualization vis = item.getVisualization();
			TupleSet ts = vis.getFocusGroup(group);

			if (item != curFocus) {

				boolean ctrl = e.isControlDown();

				if (!ctrl) {
					curFocus = item;
					ts.setTuple(item);
				} else if (ts.containsTuple(item)) {
					ts.removeTuple(item);
				} else {
					ts.addTuple(item);
				}

			} else if (e.isControlDown()) {

				ts.removeTuple(item);
				curFocus = null;

			}

			runActivity(vis);
			item.getVisualization().repaint();
			
			action.runPreviewAdjacentNodesAction(item);
		}

		// dynamically set the level of opened levels 
		// all the selected levels should stay open as long as possible
		// to avoid repainting of the tree
		int depth = tree.getDepth(item.getRow());

		if (fisheye != null && depth != -1
				&& fisheye.getDistance() < (depth + 1)) {
			fisheye.setDistance(depth + 1);
		}
		if (fisheye != null && fisheye.getDistance() == 0) {
			fisheye.setDistance(2);
		}
	}

	/**
	 * This method builds the corresponding subtree for the selected item
	 * according to its given edges in the graph. <br>
	 * Variations:
	 * <ul>
	 * <li>there are no items in the subtree, only the parent of the node is in the tree</li>
	 * <li>items in the subtree are visible. remove them from the other subtrees 
	 * as long as they are not a root of the direct parents to this nodes and add them as childs to
	 * the current node. (animate the fly-over)</li>
	 * <li>items in the subtree are invisible. just add them and the belonging edges to the tree</li>
	 * </ul>
	 * 
	 * @param item
	 *            the VisualItem which was currently selected, so the node where
	 *            to start building the subtree from the graph-file
	 */
	private void buildSubtreeFromGraph(VisualItem item) {
		
		if (item instanceof NodeItem) {

			// check if the item is a regular node or a dummy node
			if (item.getRow() >= runner.getGraph().getNodeCount())
				return;
			
			Node source = tree.getNode(item.getRow());

			Graph g = runner.getGraph();
			Node graphNode = g.getNode(source.getRow());

			// Search all incoming and outgoing edges in the graph
			List<Node> in = TreePlusGraphHelper.findAdjacentNodes(g.inEdges(graphNode), graphNode);

			List<Node> out = TreePlusGraphHelper.findAdjacentNodes(g.outEdges(graphNode), graphNode);

			// for painting the arrows, setting the adjacent nodes to
			// hightlight them later on in the visualization
			Dictionary<Node, Direction> nodes = new Hashtable<Node, Direction>();
			runner.setAdjacentNodesOfSelectedItem(nodes);

			// now process all found nodes according to their visibility etc.
			processNodeList(g, source, in, Direction.In, nodes);
			processNodeList(g, source, out, Direction.Out, nodes);

			// add the dummy nodes to the vis
			addMovedInformation(source);
		}
	}

	/**
	 * Adds the dummy nodes like "+ 6 moved" to the tree.
	 * This method uses an underlying data structure to determine, if a new dummy node has to be
	 * created or an old one has to be reused by updating the number of moved nodes.
	 * @param source indicates the currently selected node 
	 */
	private void addMovedInformation(Node source) {

		// if the currently selected source node is in the list
		// we can for sure remove it, cause all the subitems are rendered 
		if (movedNodeIdsPerNode.containsKey(source.getRow())) {
			
			tree.removeNode(movedNodeIdsPerNode.get(source.getRow()));
			movedNodeIdsPerNode.remove(source.getRow());
			movedNodesCountPerNode.remove(source.getRow());
			
			runner.clearMovedNodesForSourceNode(source.getRow());
		}	

		// now go through all the other moved infos and render the dummy nodes
		// or update existing dummy nodes in the visualization
		for (Entry<Integer, Integer> entry : movedNodesCountPerNode.entrySet()) {

			// in some special cases items moved from one subtree already
			// displaying information about how many nodes moved to another.
			// this means, that a dummy node would probably display the value 0
			// these nodes can be removed / skipped
			if (entry.getValue() == 0) {
				
				if (movedNodeIdsPerNode.containsKey(entry.getKey())) {
					System.out.println("remove node");
					tree.removeNode(movedNodeIdsPerNode.get(entry.getKey()));
					movedNodeIdsPerNode.remove(entry.getKey());
				}

				continue;
			}

			Node node = null;
			boolean inCache = movedNodeIdsPerNode.containsKey(entry.getKey());

			// if there is already a dummy node for the parent, reuse it
			if (inCache)
				node = tree.getNode(movedNodeIdsPerNode.get(entry.getKey()));
			else
				node = tree.addNode();
			
			Node father = tree.getNode(entry.getKey());
			
			if (!node.neighbors().hasNext() 
					&& movedNodeIdsPerNode.containsKey(entry.getKey())) {
				
				// if a whole subtree is closed and only the number of moved
				// items would remain, we simply remove this information as well.
				// it would just distract the user.
								
				tree.removeNode(movedNodeIdsPerNode.get(entry.getKey()));
				movedNodeIdsPerNode.remove(entry.getKey());

				entry.setValue(0);
				
			} else {
				
				// otherwise use the number of moved nodes to display the 
				// additional information
				
				node.setString(runner.getLabel(), "+ " + entry.getValue()
						+ " moved");
				
				// if the node is a new node, we have to insert it into the tree
				if (!inCache || tree.getEdge(father, node) == null)
					tree.addEdge(father, node);
				
				// add the node to the case
				movedNodeIdsPerNode.put(father.getRow(), node.getRow());
				
				// set the relationship between dummy node and the parent
				// to which it belongs
				runner.setSourceNodeForDummyNode(node.getRow(), entry.getKey());
			}
		}

	}

	
	/**
	 * Processes all the adjacent nodes to the source node by determining, if they are visible,
	 * invisible, a parent or already shown. Moreover this method fills the given nodes dictionary 
	 * according to the information of in, out or bi adjacent nodes.
	 * <p>
	 * The processing is done in the following order.
	 * <ol>
	 * <li>first of all, all the parents of the (direct and indirect) of the current selected node 
	 * are determined. This is needed to NOT remove one of them from the existing structure. This
	 * would lead to a complete destroyed tree or a restructuring. </li>
	 * <li>run through all the rest of the nodes and move them to the currently selected node. There
	 * are now two cases: the nodes are visible or invisible
	 * <ul>
	 * 	<li>invisible nodes can simply be added to the new node</li>
	 *  <li>visble nodes: they have to be moved from their current parent to their new parent - 
	 *  the current selected node. Therefore, add a counter to the original parents to count how
	 *  many items are moved.</li>
	 * </ul>
	 * </li>
	 * </ol>
	 * </p>
	 * @param graph the graph from which to determine the adjacent nodes
	 * @param source the selected new source
	 * @param nodeList the node list of adjacent nodes
	 * @param dir the current direction of the nodes (in, out, bi)
	 * @param nodes the dictionary where to store the information about the nodes (if they
	 * are in, out, or bi)
	 */
	private void processNodeList(Graph graph, Node source, List<Node> nodeList,
			Direction dir, Dictionary<Node, Direction> nodes) {

		List<Integer> parents = new ArrayList<Integer>();

		Node parent = source.getParent();

		// determine the parents of the node to avoid removing them from the list
		if (parent != null) {
			do {
				int parentIndex = parent.getRow();
				parents.add(parentIndex);
			} while ((parent = parent.getParent()) != null);
		}

		// now run through all the adjacent nodes and move them to the new selected node
		for (Node node : nodeList) {

			if (TreePlusGraphHelper.isBidirectionalAdjacent(graph, source
					.getRow(), node.getRow()))
				nodes.put(node, Direction.Bi);
			else
				nodes.put(node, dir);

			Node target = tree.getNode(node.getRow());

			// be aware of the structure of the tree, do not
			// remove the root reference from a subtree to a root node
			if (parents.contains(target.getRow())) {
				continue;
			}

			// move the node from the old parent to the new parent --> the source
			processSubtree(source, target);
		}
	}

	/**
	 * In here the given target node is being checked, if it is visible or not and
	 * if it has already a parent from which it has to be removed. If so, add a counter to the
	 * parent to determine how many childs moved.
	 * @param source the source node
	 * @param target the new target node (child of source)
	 */
	private void processSubtree(Node source, Node target) {

		if (source == target) {
			// this would be a circular reference. --> avoid adding
			// example: foodweb --> beetles eat beetles
			return;
		}

		Iterator<Edge> edges = target.edges();
		List<Integer> indizes = new ArrayList<Integer>();

		// check if the node is now a moved node are a newly inserted
		CheckMoved(source, target);

		while (edges != null && edges.hasNext()) {
			Edge edge = edges.next();
			indizes.add(edge.getRow());
		}

		for (Integer integer : indizes) {
			tree.removeEdge(integer);
		}

		tree.addEdge(source, target);
	}

	/**
	 * checks, if the target node is a new inserted node into the vis or if it 
	 * was already a child of another node in the vis. if so, add a counter for the old parent node.
	 * @param source the source node
	 * @param target the target node
	 */
	private void CheckMoved(Node source, Node target) {

		if (target.getParent() != null) {
			
			Node parent = target.getParent();

			// if the parent is not visible, the node is not visible as well
			// and we do not have to do anything. just add them to the new source
			if (parent != source
					&& TreePlusGraphHelper.isNodeVisible(tree, parent)) {
				
				// if the parent is already opened (shown) in the tree
				// we have to add the moved info.
				
				int row = parent.getRow();
				int count = 0;

				// look if there is already an item in the cache
				if (movedNodesCountPerNode.containsKey(row)) {
					count = movedNodesCountPerNode.get(row);
				}

				// add a moved node
				movedNodesCountPerNode.put(row, ++count);				
				
				// add this info to highlight the corresponding moved nodes in the vis
				// later on
				runner.addMovedNodeForSourceNode(row, target.getRow());

				row = source.getRow();
				count = 0;

				// the new source node has a moved node, it is possible that
				// the newly inserted child was removed by another action a few steps 
				// before. so we have to check this and in case decrement the counter
				// for the source node
				if (movedNodesCountPerNode.containsKey(row)) {
					count = movedNodesCountPerNode.get(row);
					count--;

					if (count < 0)
						count = 0;					
				}

				// do this also in the highlight infos
				runner.removeMovedNodeForSourceNode(row, target.getRow());
				
				movedNodesCountPerNode.put(row, count);
			}
		}
	}

	/**
	 * runs the given activity on the visualization.
	 * @param vis the visualization, which should run the activity
	 */
	private void runActivity(Visualization vis) {
		if (activity != null) {
			vis.run(activity);		
		}
	}

}
