package treeplus.visualization;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.visual.VisualItem;

/**
 * Helper class used to determine various attributes of nodes in the underlying graph.
 * @author Neubauer, Kstinger
 *
 */
public final class TreePlusGraphHelper {

	/**
	 * Finds all adjacent nodes in the {@link edges} list to the current node.
	 * @param edges the edges to check
	 * @param currentNode the current node, to find the adjacent nodes for
	 * @return a list of adjacent nodes to the current node
	 */
	public static List<Node> findAdjacentNodes(Iterator<Edge> edges, Node currentNode) {
    	
    	List<Node> adjacentNodes = new ArrayList<Node>();
    	
		while(edges != null && edges.hasNext()) {
			Edge edge = edges.next();
			
			Node adjacent = edge.getAdjacentNode(currentNode);			
			adjacentNodes.add(adjacent);			
		}
		
		return adjacentNodes;
    }
	
	/**
	 * checks whether the given node is visible in the visualization or not. 
	 * @param tree the graph, to which the node belongs
	 * @param node the node to check
	 * @return true - if the node is visible, otherwise false
	 */
	public static boolean isNodeVisible(Graph tree, Node node) {
		
		if (tree.getNode(node.getRow()).canGetBoolean(VisualItem.VISIBLE))
			return tree.getNode(node.getRow()).getBoolean(VisualItem.VISIBLE);
		
		return true;
	}

	/**
	 * checks, if the two given nodes (determined by their index positions) are bidirectional
	 * adjacent to one another. Two nodes are bidirectional adjacent, if there are edges from 
	 * the source to the target as well as edges from the target to the source node.
	 * @param graph the graph in which to check
	 * @param source the source node
	 * @param target the target node
	 * @return true - if the nodes are bidirectional adjacent
	 */
	public static boolean isBidirectionalAdjacent(Graph graph, int source, int target) {
		Node sourceNode = graph.getNode(source);
		Node targetNode = graph.getNode(target);
		
		return isBidirectionalAdjacent(graph, sourceNode, targetNode);
	}
	
	/**
	 * checks, if the two given nodes (determined by their index positions) are bidirectional
	 * adjacent to one another. Two nodes are bidirectional adjacent, if there are edges from 
	 * the source to the target as well as edges from the target to the source node.
	 * @param graph the graph in which to check
	 * @param source the source node
	 * @param target the target node
	 * @return true - if the nodes are bidirectional adjacent
	 */
	public static boolean isBidirectionalAdjacent(Graph graph, Node source, Node target) {
		boolean biAdjacent = false;
		
		biAdjacent = (graph.getEdge(source, target) != null) &&
					 (graph.getEdge(target, source) != null);
		
		return biAdjacent;
	}
}
