package treeplus.visualization.demo;

import java.util.Dictionary;
import java.util.List;
import java.util.Set;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JFrame;

import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.visual.VisualItem;
import treeplus.visualization.DegreeHelper;
import treeplus.visualization.Direction;

public interface ITreePlusRunner {
	
	/**
	 * Sets a new graph for the visualization.
	 * @param graph the new graph which will be visualized as tree
	 * @param label the new name of the nodes in the graph
	 */
	void setGraph(Graph graph, String label);
	
	/**
	 * @return the visualized graph
	 */
	Graph getGraph();
	
	/**
	 * @return the main JFrame of the program
	 */
	JFrame getEnclosingFrame();
	
	/**
	 * Sets the given node as root in the tree.
	 * @param node the new root 
	 */
	void setSelectedGraphNodeForSpanningTree(Node node);

	/**
	 * Returns the tooltip for a node.
	 * The tooltip includes the name of the node, the number of incoming, outgoing 
	 * and bidirectional edges of the node.
	 * @param row the row of the node to get the tooltip for
	 * @return the tooltip for the node in the given row
	 */
	String getToolTipForNodeRow(int row);
	
	/**
	 * Returns the degree of the node in the given row
	 * @param row the row of the nodes
	 * @return a DregreeHelper with the number of incoming, outgoing and bidirectional edges of the node
	 */
	DegreeHelper getDegree(int row);
	
	/**
	 * Returns a list of the row indices of moved nodes of the given source node.
	 * @param source the row index of the parent node
	 * @return a list of row indices with moved nodes of the given source node
	 */
	List<Integer> getMovedNodesForSourceNode(int source);
	
	/**
	 * Adds a node to the list of moved nodes of the given source node.
	 * @param source the row index of the source node
	 * @param movedNode the row index of the node to add to the list of moved nodes
	 */
	void addMovedNodeForSourceNode(int source, int movedNode);
	
	/**
	 * Removes a moved node from the list of moved nodes of the given source node.
	 * @param source the row index of the source node
	 * @param movedNode the row index of the node which will be removed from the list of the moved nodes
	 */
	void removeMovedNodeForSourceNode(int source, int movedNode);
	
	/**
	 * Removes the list of moved nodes of the given source node.
	 * @param source the row index of the source node 
	 */
	void clearMovedNodesForSourceNode(int source);
	
	/**
	 * Returns a set of keys of the 
	 */
	Set<Integer> getSourceNodes();
	
	/**
	 * Sets a new relation between a source and a dummy node
	 * @param dummy the row index of the dummy node which represents the number of moved nodes 
	 * @param source the row index of the parent node of the dummy node
	 */
	void setSourceNodeForDummyNode(int dummy, int source);
	
	/**
	 * Returns the row index of the parent node of the given dummy node
	 * @param dummy the row index of the dummy node
	 * @return the row index of the parent node of the given dummy node
	 */
	int getSourceNodeForDummyNode(int dummy);
	
	/**
	 * Removes the dummy node from the list of all dummy nodes.
	 * @param dummy the index key of the dummy node to remove
	 */
	void clearSourceNodeForDummyNode(int dummy);
	
	/**
	 * Returns the file name of the image with the bidirectional arrow.
	 * @return the file name of the image with the bidirectional arrow
	 */
	String getBidirectionalFile();
	
	/**
	 * Returns the file name of the image with the red arrow which points to the right.
	 * @return the file name of the image with the red-right arrow
	 */
	String getRedRightFile();
	
	/**
	 * Returns the file name of the image with the blue arrow which points to the right.
	 * @return the file name of the image with the blue-right arrow
	 */
	String getBlueRightFile();
	
	/**
	 * Returns the file name of the image with the red arrow which points to the left.
	 * @return the file name of the image with the red-left arrow
	 */
	String getRedLeftFile();
	
	/**
	 * Returns the file name of the image with the blue arrow which points to the left.
	 * @return the file name of the image with the blue-left arrow
	 */
	String getBlueLeftFile();
	
	/**
	 * Returns the Icon for the legend to visualize outgoing edges.
	 * @return the Icon for outgoing edges 
	 */
	Icon getLegendIconBlue();
	
	/**
	 * Returns the Icon for the legend to visualize incoming edges.
	 * @return the Icon for incoming edges
	 */
	Icon getLegendIconRed();
	
	/**
	 * Returns the Icon for the legend to visualize bidirectional relations between nodes
	 * @return the Icon for bidirectional relations 
	 */
	Icon getLegendIconBi();
	
	/**
	 * Returns the Icon for the legend to visualize the actual active node.
	 * @return the Icon for the active node 
	 */
	Icon getLegendIconActive();
	
	/**
	 * Returns the Icon for the legend to visualize nodes which were found by searching.
	 * @return the Icon for nodes which were found by searching
	 */
	Icon getLegendIconSearch();
	
	/**
	 * Returns the direction of the node with the given nodeName if it is in the list of 
	 * visible adjacent nodes of the actual active item. If not null is returned.
	 * @param nodeName the name of node to get the direction
	 * @return the direction of the node with the given node name
	 */
	Direction isInVisibleNodesOfClickedItem(String nodeName);
	
	/**
	 * Sets the list of moved item to the given list.
	 * @param movedList the list of moved items
	 */
	void setMovedListOfItem(List<Integer> movedList);
	
	/**
	 * Sets the actual visible adjacent nodes to the given node list.
	 * @param nodes the actual visible adjacent nodes of the clicked item
	 */
	void setAdjacentNodesOfSelectedItem(Dictionary<Node, Direction> visibleNodes);
	
	/**
	 * Returns the actual list of moved items.
	 * @return the list of moved items
	 */
	List<Integer> getMovedItemsList();
	
	/**
	 * Returns the actual active item.
	 * @return the active item
	 */
	VisualItem getActiveItem();
	
	/**
	 * Sets the actual active item to the given item.
	 * @param item the actual clicked item
	 */
	void setActiveItem(VisualItem item);
	
	/**
	 * @return the label of the nodes
	 */
	String getLabel();
	
	/**
	 * @return the view of the tree
	 */
	JComponent getActualTreeView();
	
	/**
	 * @return the view of the graph
	 */
	JComponent getActualGraphView();
	
	/**
	 * Sets a new graph view.
	 * @param graphView the new graphView
	 */
	void setActualGraphView(JComponent graphView);
}
