package treeplus.visualization.demo;

import java.net.URL;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Tree;
import prefuse.data.io.GraphMLReader;
import prefuse.util.GraphLib;
import prefuse.visual.VisualItem;
import treeplus.visualization.DegreeHelper;
import treeplus.visualization.Direction;
import treeplus.visualization.OpenGraphAction;
import treeplus.visualization.SpanningTreePlus;
import treeplus.visualization.SwitchViewAction;
import treeplus.visualization.TreePlus;
import treeplus.visualization.TreePlusGraphHelper;

/**
 * The TreePlusRunner creates a new JFrame which includes a new TreePlus instance.
 * It also supports some useful methods for the visualization, e.g. a method to get the tooltip for a special node.
 * @author Neubauer, Kstinger
 *
 */
public class TreePlusRunner implements ITreePlusRunner {

	private JFrame mainView;
	JPanel contentPane = new JPanel();
	private JComponent treeView;
	private JComponent graphView;
	private Graph graph = null;
	private String label = null;;

	private Hashtable<Integer, List<Integer>> movedNodesPerSourceNode;
	private Hashtable<Integer, Integer> sourceNodeForDummyNode;

	private String redRightFile = "/img/red_ToRight.gif";
	private String redLeftFile = "/img/red_ToLeft.gif";
	private String blueRightFile = "/img/blue_ToRight.gif";
	private String blueLeftFile = "/img/blue_ToLeft.gif";
	private String bidirectionalFile = "/img/bidirectional.gif";

	private String legendBlue = "/img/legendBlue.gif";
	private String legendRed = "/img/legendRed.gif";
	private String legendBi = "/img/legendBi.gif";
	private String legendActive = "/img/legendActive.gif";
	private String legendSearch = "/img/legendSearch.gif";

	private Dictionary<Node, Direction> visibleNodes;
	private List<Integer> movedList;

	private VisualItem activeItem;
	private SwitchViewAction switchView;
	
	/**
	 * Creates a new TreePlus with the given graph file. 
	 * The given label defines the name of the nodes which are used in the Tree.
	 * @param graphfile the file with the graph to visualize
	 * @param label the name of the nodes
	 */
	public void run(String graphfile, String label) {

		movedNodesPerSourceNode = new Hashtable<Integer, List<Integer>>();
		sourceNodeForDummyNode = new Hashtable<Integer, Integer>();

		if (graphfile == null) {
			graph = GraphLib.getGrid(15, 15);
			label = "label";
		} else {
			try {
				graph = new GraphMLReader().readGraph(graphfile);
			} catch (Exception e) {
				e.printStackTrace();
				System.exit(1);
			}
		}

		this.label = label;

		Tree t = new SpanningTreePlus(graph, label);

		treeView = TreePlus.demo(t, label, this);

		JMenuBar menuBar = new JMenuBar();

		JMenu menu = new JMenu("Data");
		menu.add(new OpenGraphAction(this));
		menuBar.add(menu);

		JMenu menuVis = new JMenu("Visualization");
		switchView = new SwitchViewAction(this);
		menuVis.add(switchView);
		menuBar.add(menuVis);

		JFrame frame = new JFrame(
				"i n f o v i s  |  g r a p h s    a s    t r e e s");
		frame.setJMenuBar(menuBar);
		frame.setContentPane(contentPane);
		frame.setContentPane(treeView);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);

		mainView = frame;

		int startNodeIndex = this.getStartNodeIndex(graph);
		if (startNodeIndex != -1) {
			Node node = graph.getNode(startNodeIndex);
			setSelectedGraphNodeForSpanningTree(node);
		}
	}
	/**
	 * @return the label of the nodes
	 */
	public String getLabel() {
		return this.label;
	}

	/**
	 * @return the main JFrame of the program
	 */
	public JFrame getEnclosingFrame() {
		return mainView;
	}

	/**
	 * @return the visualized graph
	 */
	public Graph getGraph() {
		return graph;
	}

	/**
	 * Sets a new graph for the visualization.
	 * @param graph the new graph which will be visualized as tree
	 * @param label the new name of the nodes in the graph
	 */
	public void setGraph(Graph graph, String label) {
		this.label = label;
		this.graph = graph;
		this.visibleNodes = null;
		Node node = graph.getNode(getStartNodeIndex(graph));
		setSelectedGraphNodeForSpanningTree(node);
	}

	/**
	 * Returns the node with the most edges. 
	 * This node is used as the initial root node in the tree.
	 * @param graph the graph which includes all nodes
	 * @return the index of the node with the most edges
	 */
	private int getStartNodeIndex(Graph graph) {
		int maxDegree = Integer.MIN_VALUE;
		int index = 0;
		for (int i = 0; i < graph.getNodeCount(); i++) {
			Node n = graph.getNode(i);
			if (maxDegree < n.getDegree()) {
				index = n.getRow();
				maxDegree = n.getDegree();
			}
		}
		return index;
	}

	/**
	 * Sets the given node as root in the tree.
	 * @param node the new root 
	 */
	public void setSelectedGraphNodeForSpanningTree(Node node) {
		
		movedNodesPerSourceNode = new Hashtable<Integer, List<Integer>>();
		sourceNodeForDummyNode = new Hashtable<Integer, Integer>();
		
		switchView.switchName(SwitchViewAction.SWITCH_TO_GRAPH);
		Tree t = null;
		try {
			Graph g = node.getGraph();
			t = new SpanningTreePlus(g, node, label);
			t.setRoot(t.getNode(node.getRow()));
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		treeView = TreePlus.demo(t, label, this);

		mainView.setContentPane(treeView);
		mainView.validate();
		mainView.repaint();
	}

	/**
	 * @return the view of the graph
	 */
	public JComponent getActualGraphView() {
		return this.graphView;
	}
	
	/**
	 * Sets a new graph view.
	 * @param graphView the new graphView
	 */
	public void setActualGraphView(JComponent graphView){
		this.graphView = graphView;
	}

	/**
	 * @return the view of the tree
	 */
	public JComponent getActualTreeView() {
		return this.treeView;
	}

	/**
	 * Returns the tooltip for a node.
	 * The tooltip includes the name of the node, the number of incoming, outgoing 
	 * and bidirectional edges of the node.
	 * @param row the row of the node to get the tooltip for
	 * @return the tooltip for the node in the given row
	 */
	public String getToolTipForNodeRow(int row) {

		if (row >= graph.getNodeCount())
			return null;

		Node node = graph.getNode(row);

		if (node == null)
			return null;

		String tooltip = "";

		if (node.canGetString(label))
			tooltip += node.getString(label);

		if (tooltip != null) {
			tooltip += addDegreeInfo(node);
		}

		return tooltip;
	}

	/**
	 * Returns a String which contains the number of incoming, outgoing 
	 * and bidirectional edges of the given node.
	 * @param node the node to get the number of edges
	 * @return a String which contains the number of edges
	 */
	private String addDegreeInfo(Node node) {
		String tooltip = "";
		
		DegreeHelper helper = calcDegreeLevels(node);

		tooltip += ", out: " + helper.getOut();
		tooltip += ", bi: " + helper.getBi();
		tooltip += ", in: " + helper.getIn();
		
		return tooltip;
	}
	
	/**
	 * Calculates the incoming, outgoing and bidirectional edges of the given node.
	 * @param node the node for calculating the edges
	 * @return the DegreeHelper which includes the information about the incoming, outgoing and bidirectional edges
	 */
	private DegreeHelper calcDegreeLevels(Node node) {

		int biCount = 0;
		int inCount = node.getInDegree();
		int outCount = node.getOutDegree();

		Iterator<Edge> inEdges = graph.inEdges(node);
		List<Node> in = TreePlusGraphHelper.findAdjacentNodes(inEdges, node);

		Iterator<Edge> outEdges = graph.outEdges(node);
		List<Node> out = TreePlusGraphHelper.findAdjacentNodes(outEdges, node);

		for (Node outNode : out) {
			if (TreePlusGraphHelper.isBidirectionalAdjacent(graph, node, outNode)) {

				biCount++;
				outCount--;
			}
		}
		
		in.removeAll(out);
		
		for (Node inNode : in) {
			if (TreePlusGraphHelper.isBidirectionalAdjacent(graph, node, inNode)) {

				biCount++;
				inCount--;
			}
		}

		return new DegreeHelper(outCount, biCount, inCount);
	}

	/**
	 * Returns the number of incoming edges of the node in the given row.
	 * @param row the row of the node
	 * @return the number of incoming edges
	 */
	public int getInDegree(int row) {
		if (row >= graph.getNodeCount())
			return 0;

		Node node = graph.getNode(row);
		return node.getInDegree();
	}

	/**
	 * Returns the number of outgoing edges of the node in the given row.
	 * @param row the row of the node
	 * @return the number of outgoing edges
	 */
	public int getOutDegree(int row) {
		if (row >= graph.getNodeCount())
			return 0;

		Node node = graph.getNode(row);
		return node.getOutDegree();
	}

	
	/**
	 * Adds a node to the list of moved nodes of the given source node.
	 * @param source the row index of the source node
	 * @param movedNode the row index of the node to add to the list of moved nodes
	 */
	public void addMovedNodeForSourceNode(int source, int movedNode) {

		List<Integer> list = null;

		if (movedNodesPerSourceNode.containsKey(source))
			list = movedNodesPerSourceNode.get(source);
		else
			list = new ArrayList<Integer>();

		list.add(movedNode);

		if (!movedNodesPerSourceNode.containsKey(source))
			movedNodesPerSourceNode.put(source, list);
	}

	/**
	 * Removes a moved node from the list of moved nodes of the given source node.
	 * @param source the row index of the source node
	 * @param movedNode the row index of the node which will be removed from the list of the moved nodes
	 */
	public void removeMovedNodeForSourceNode(int source, int movedNode) {
		if (movedNodesPerSourceNode.containsKey(source)) {
			List<Integer> list = movedNodesPerSourceNode.get(source);

			if (list != null && list.indexOf(movedNode) != -1)
				list.remove(list.indexOf(movedNode));
		}
	}

	/**
	 * Removes the list of moved nodes of the given source node.
	 * @param source the row index of the source node 
	 */
	public void clearMovedNodesForSourceNode(int source) {
		movedNodesPerSourceNode.remove(source);
	}

	/**
	 * Returns a list of the row indices of moved nodes of the given source node.
	 * @param source the row index of the parent node
	 * @return a list of row indices with moved nodes of the given source node
	 */
	public List<Integer> getMovedNodesForSourceNode(int source) {
		if (movedNodesPerSourceNode.containsKey(source))
			return movedNodesPerSourceNode.get(source);

		return new ArrayList<Integer>();
	}

	/**
	 * Returns the row index of the parent node of the given dummy node
	 * @param dummy the row index of the dummy node
	 * @return the row index of the parent node of the given dummy node
	 */
	public int getSourceNodeForDummyNode(int dummy) {

		if (!sourceNodeForDummyNode.containsKey(dummy))
			return -1;

		return sourceNodeForDummyNode.get(dummy);
	}

	/**
	 * Sets a new relation between a source and a dummy node
	 * @param dummy the row index of the dummy node which represents the number of moved nodes 
	 * @param source the row index of the parent node of the dummy node
	 */
	public void setSourceNodeForDummyNode(int dummy, int source) {
		sourceNodeForDummyNode.put(dummy, source);
	}

	/**
	 * Returns a set of keys of the 
	 */
	public Set<Integer> getSourceNodes() {

		if (movedNodesPerSourceNode != null)
			return movedNodesPerSourceNode.keySet();

		return null;
	}

	/**
	 * Removes the dummy node from the list of all dummy nodes.
	 * @param dummy the index key of the dummy node to remove
	 */
	public void clearSourceNodeForDummyNode(int dummy) {
		sourceNodeForDummyNode.remove(dummy);
	}

	/**
	 * Returns the file name of the image with the red arrow which points to the left.
	 * @return the file name of the image with the red-left arrow
	 */
	public String getRedLeftFile() {
		return this.redLeftFile;
	}

	/**
	 * Returns the file name of the image with the red arrow which points to the right.
	 * @return the file name of the image with the red-right arrow
	 */
	public String getRedRightFile() {
		return this.redRightFile;
	}

	/**
	 * Returns the file name of the image with the blue arrow which points to the right.
	 * @return the file name of the image with the blue-right arrow
	 */
	public String getBlueRightFile() {
		return this.blueRightFile;
	}

	/**
	 * Returns the file name of the image with the blue arrow which points to the left.
	 * @return the file name of the image with the blue-left arrow
	 */
	public String getBlueLeftFile() {
		return this.blueLeftFile;
	}
	
	/**
	 * Returns the file name of the image with the bidirectional arrow.
	 * @return the file name of the image with the bidirectional arrow
	 */
	public String getBidirectionalFile() {
		return bidirectionalFile;
	}

	/**
	 * Returns the direction of the node with the given nodeName if it is in the list of 
	 * visible adjacent nodes of the actual active item. If not null is returned.
	 * @param nodeName the name of node to get the direction
	 * @return the direction of the node with the given node name
	 */
	public Direction isInVisibleNodesOfClickedItem(String nodeName) {
		if (visibleNodes != null && nodeName != null) {
			for (Enumeration<Node> keys = visibleNodes.keys(); keys
					.hasMoreElements();) {
				Node node = keys.nextElement();
				if (node.getString(label).equals(nodeName)) {
					return visibleNodes.get(node);
				}
			}
		}
		return null;
	}

	/**
	 * Sets the list of moved item to the given list.
	 * @param movedList the list of moved items
	 */
	public void setMovedListOfItem(List<Integer> movedList) {
		this.movedList = movedList;
	}

	/**
	 * Sets the actual visible adjacent nodes to the given node list.
	 * @param nodes the actual visible adjacent nodes of the clicked item
	 */
	public void setAdjacentNodesOfSelectedItem(Dictionary<Node, Direction> nodes) {
		this.visibleNodes = nodes;
	}

	/**
	 * Returns the actual list of moved items.
	 * @return the list of moved items
	 */
	public List<Integer> getMovedItemsList() {
		return this.movedList;
	}

	/**
	 * Returns the actual active item.
	 * @return the active item
	 */
	public VisualItem getActiveItem() {
		return this.activeItem;
	}

	/**
	 * Sets the actual active item to the given item.
	 * @param item the actual clicked item
	 */
	public void setActiveItem(VisualItem item) {
		this.activeItem = item;
		item.getVisualization().run("repaint");
	}

	/**
	 * Returns the Icon for the legend to visualize outgoing edges.
	 * @return the Icon for outgoing edges 
	 */
	public Icon getLegendIconBlue() {
		return getIcon(legendBlue);
	}

	/**
	 * Returns the Icon for the legend to visualize incoming edges.
	 * @return the Icon for incoming edges
	 */
	public Icon getLegendIconRed() {
		return getIcon(legendRed);
	}

	/**
	 * Returns the Icon for the legend to visualize the actual active node.
	 * @return the Icon for the active node 
	 */
	public Icon getLegendIconActive() {
		return getIcon(legendActive);
	}

	/**
	 * Returns the Icon for the legend to visualize nodes which were found by searching.
	 * @return the Icon for nodes which were found by searching
	 */
	public Icon getLegendIconSearch() {
		return getIcon(legendSearch);
	}
	
	/**
	 * Returns the Icon for the legend to visualize bidirectional relations between nodes
	 * @return the Icon for bidirectional relations 
	 */
	public Icon getLegendIconBi() {
		return getIcon(legendBi);
	}
	
	/**
	 * Loads the icon with the given filename and returns it.
	 * @param iconFile the filename of the icon
	 * @return the Icon with the given filename
	 */
	private Icon getIcon(String iconFile) {
		URL imgURL = getClass().getResource(iconFile);
		if (imgURL != null) {
			ImageIcon icon = new ImageIcon(imgURL);
			return icon;
		}
		return null;
	}

	
	/**
	 * Returns the degree of the node in the given row
	 * @param row the row of the nodes
	 * @return a DregreeHelper with the number of incoming, outgoing and bidirectional edges of the node
	 */
	public DegreeHelper getDegree(int row) {
		
		if (row >= graph.getNodeCount())
			return null;
		
		return calcDegreeLevels(graph.getNode(row));
	}

}
